==> aspectslib-1.1.2-sources.jar: dev/overgrown/aspectslib/resonance/ResonanceCalculator.java <==
package dev.overgrown.aspectslib.resonance;

import dev.overgrown.aspectslib.data.AspectData;
import net.minecraft.util.Identifier;

import java.util.Collections;
import java.util.List;
import java.util.Map;

public class ResonanceCalculator {
    public static ResonanceResult calculate(AspectData data) {
        double totalRU = 0;
        double amplificationFactor = 1.0;
        double barrierCost = 0;

        Map<Identifier, Integer> aspects = data.getMap();

        for (Map.Entry<Identifier, Integer> entry : aspects.entrySet()) {
            Identifier aspectId = entry.getKey();
            int amount = entry.getValue();

            // Get all resonance relationships for this aspect
            List<Resonance> resonances = ResonanceManager.RESONANCE_MAP.getOrDefault(
                    aspectId, Collections.emptyList()
            );

            for (Resonance resonance : resonances) {
                if (resonance == null) continue;

                // Find the other aspect in the pair
                Identifier otherId = resonance.aspect1().equals(aspectId) ?
                        resonance.aspect2() : resonance.aspect1();

                if (aspects.containsKey(otherId)) {
                    int otherAmount = aspects.get(otherId);

                    if (resonance.type() == Resonance.Type.AMPLIFYING) {
                        // Amplification: average of both amounts * factor
                        double boost = ((amount + otherAmount) / 2.0) * resonance.factor();
                        amplificationFactor += boost;
                        totalRU += boost;
                    } else { // OPPOSING
                        // Barrier cost: min amount * factor
                        double barrier = Math.min(amount, otherAmount) * resonance.factor();
                        barrierCost += barrier;
                        totalRU -= barrier;
                    }
                }
            }
        }

        return new ResonanceResult(totalRU, amplificationFactor, barrierCost);
    }

    public record ResonanceResult(double totalRU, double amplificationFactor, double barrierCost) {}
}

==> aspectslib-1.1.5-sources.jar: dev/overgrown/aspectslib/resonance/ResonanceCalculator.java <==
package dev.overgrown.aspectslib.resonance;

import dev.overgrown.aspectslib.data.AspectData;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import net.minecraft.class_2960;

public class ResonanceCalculator {
    public static ResonanceResult calculate(AspectData data) {
        double totalRU = 0;
        double amplificationFactor = 1.0;
        double barrierCost = 0;

        Map<class_2960, Integer> aspects = data.getMap();

        for (Map.Entry<class_2960, Integer> entry : aspects.entrySet()) {
            class_2960 aspectId = entry.getKey();
            int amount = entry.getValue();

            // Get all resonance relationships for this aspect
            List<Resonance> resonances = ResonanceManager.RESONANCE_MAP.getOrDefault(
                    aspectId, Collections.emptyList()
            );

            for (Resonance resonance : resonances) {
                if (resonance == null) continue;

                // Find the other aspect in the pair
                class_2960 otherId = resonance.aspect1().equals(aspectId) ?
                        resonance.aspect2() : resonance.aspect1();

                if (aspects.containsKey(otherId)) {
                    int otherAmount = aspects.get(otherId);

                    if (resonance.type() == Resonance.Type.AMPLIFYING) {
                        // Amplification: average of both amounts * factor
                        double boost = ((amount + otherAmount) / 2.0) * resonance.factor();
                        amplificationFactor += boost;
                        totalRU += boost;
                    } else { // OPPOSING
                        // Barrier cost: min amount * factor
                        double barrier = Math.min(amount, otherAmount) * resonance.factor();
                        barrierCost += barrier;
                        totalRU -= barrier;
                    }
                }
            }
        }

        return new ResonanceResult(totalRU, amplificationFactor, barrierCost);
    }

    public record ResonanceResult(double totalRU, double amplificationFactor, double barrierCost) {}
}
